@@ -10,9 +10,10 @@ use std::str::FromStr;
 use async_trait::async_trait;
 use bytes::Bytes;
 use destream::{de, en};
-use futures::future::{try_join_all, TryFutureExt};
+use futures::future::TryFutureExt;
+use futures::stream::{self, StreamExt, TryStreamExt};
 use log::debug;
-use safecast::{TryCastFrom, TryCastInto};
+use safecast::{Match, TryCastFrom, TryCastInto};
 
 use tc_error::*;
 use tcgeneric::*;
@@ -191,7 +192,27 @@ impl Scalar {
                     .map(Scalar::Ref),
 
                 RT::Op(ort) => {
-                    if let Some(tuple) = Tuple::<Scalar>::opt_cast_from(self) {
+                    debug!("cast into op ref from {}", self);
+
+                    if ort == ORT::Delete && self.matches::<(Scalar, Scalar)>() {
+                        // TODO: is there a better way to handle this edge case?
+                        debug!("cast into DELETE ref from {}", self);
+
+                        let (subject, key): (Scalar, Scalar) = self.opt_cast_into().unwrap();
+                        let delete_ref = match subject {
+                            Scalar::Ref(tc_ref) => match *tc_ref {
+                                TCRef::Id(id_ref) => Some((id_ref.into(), key)),
+                                TCRef::Op(OpRef::Get((subject, none))) if none.is_none() => {
+                                    Some((subject, key))
+                                }
+                                _ => None,
+                            },
+                            _ => None,
+                        };
+
+                        delete_ref.map(OpRef::Delete).map(TCRef::Op).map(Self::from)
+                    } else if self.matches::<Tuple<Scalar>>() {
+                        let tuple: Tuple<Scalar> = self.opt_cast_into().unwrap();
                         debug!("cast into {} from tuple {}", ort, tuple);
 
                         let op_ref = match ort {
@@ -284,6 +305,16 @@ impl Refer for Scalar {
         }
     }
 
+    fn is_derived_write(&self) -> bool {
+        match self {
+            Self::Map(map) => map.values().any(|scalar| scalar.is_derived_write()),
+            Self::Op(op_def) => op_def.is_derived_write(),
+            Self::Ref(tc_ref) => tc_ref.is_derived_write(),
+            Self::Tuple(tuple) => tuple.iter().any(|scalar| scalar.is_derived_write()),
+            _ => false,
+        }
+    }
+
     fn is_inter_service_write(&self, cluster_path: &[PathSegment]) -> bool {
         match self {
             Self::Map(map) => map
@@ -351,27 +382,42 @@ impl Refer for Scalar {
 
         match self {
             Self::Map(map) => {
-                let resolved =
-                    try_join_all(map.into_iter().map(|(id, scalar)| {
-                        scalar.resolve(context, txn).map_ok(|state| (id, state))
-                    }))
-                    .await?;
+                let mut resolved = stream::iter(map)
+                    .map(|(id, scalar)| scalar.resolve(context, txn).map_ok(|state| (id, state)))
+                    .buffer_unordered(num_cpus::get());
 
-                Ok(State::Map(Map::from_iter(resolved)))
+                let mut map = Map::new();
+                while let Some((id, state)) = resolved.try_next().await? {
+                    map.insert(id, state);
+                }
+
+                Ok(State::Map(map))
             }
             Self::Ref(tc_ref) => tc_ref.resolve(context, txn).await,
             Self::Tuple(tuple) => {
-                let resolved =
-                    try_join_all(tuple.into_iter().map(|scalar| scalar.resolve(context, txn)))
-                        .await?;
+                let len = tuple.len();
+                let mut resolved = stream::iter(tuple)
+                    .map(|scalar| scalar.resolve(context, txn))
+                    .buffered(num_cpus::get());
 
-                Ok(State::Tuple(resolved.into()))
+                let mut tuple = Vec::with_capacity(len);
+                while let Some(state) = resolved.try_next().await? {
+                    tuple.push(state);
+                }
+
+                Ok(State::Tuple(tuple.into()))
             }
             other => Ok(State::Scalar(other)),
         }
     }
 }
 
+impl From<IdRef> for Scalar {
+    fn from(id_ref: IdRef) -> Self {
+        Self::Ref(Box::new(TCRef::Id(id_ref)))
+    }
+}
+
 impl From<Link> for Scalar {
     fn from(link: Link) -> Self {
         Value::from(link).into()
@@ -396,6 +442,12 @@ impl From<Range> for Scalar {
     }
 }
 
+impl From<TCPathBuf> for Scalar {
+    fn from(path: TCPathBuf) -> Self {
+        Self::Value(path.into())
+    }
+}
+
 impl From<TCRef> for Scalar {
     fn from(tc_ref: TCRef) -> Self {
         Self::Ref(Box::new(tc_ref))
