@@ -119,6 +119,12 @@ impl OpDef {
         .iter()
     }
 
+    pub fn is_derived_write(&self) -> bool {
+        self.form()
+            .map(|(_, provider)| provider)
+            .any(|provider| provider.is_derived_write())
+    }
+
     pub fn is_inter_service_write(&self, cluster_path: &[PathSegment]) -> bool {
         self.form()
             .map(|(_, provider)| provider)
